#!/usr/bin/python
import os
import copy
import unittest
import pyuppaal
from opaal.model_parsers.pyuppaal import GoalChecker
from opaal.model_parsers.pyuppaal.gensuccgen import GenerateSuccessorGenerator
from opaal.passed_waiting_list import SimplePWList, LatticeCoverPWList, LatticeJoinPWList
from opaal.modelchecking_algorithms.reachability import McReachability

class TestPYUPPAALGenSuccGenLattice(unittest.TestCase):
    def test_lattice_cover_bitvector(self):
        file = open(os.path.join(os.path.dirname(__file__), 'test_lattice_cover.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        succgen = GenerateSuccessorGenerator(nta)
        init_state = succgen.get_initialstate()
        print "init_state: ", init_state

        #goalchecker = GoalChecker(succgen, '')

        #pwlist = SimplePWList()
        #m = McReachability(succgen, goalchecker, pwlist)
        #pwlist.addState(init_state)
        #res = m.start()
        #self.assertEqual(7, len(pwlist.passed_waiting))

        #check that lattice inclusion check succeeds manually
        succs = list(succgen.successors(init_state))
        self.assertEqual(2, len(succs))
        succ1 = succs[0] ; succ2 = succs[1]
        print "init_state: ", init_state
        print "init_state.bv: ", init_state.bv
        print "succ1: ", succ1
        print "succ1.bv: ", succ1.bv
        print "succ2: ", succ2
        print "succ2.bv: ", succ2.bv
        self.assertFalse(succ2.discrete_eq(succ1))
        self.assertFalse(succ1.covers(succ2))
        self.assertFalse(succ2.covers(succ1))
        succ1n = list(succgen.successors(succ1))[0]
        print "succ1n: ", succ1n
        print "succ1: ", succ1
        succ2n = list(succgen.successors(succ2))[0]
        print "succ2n: ", succ2n

        self.assertTrue(succ1n.discrete_eq(succ2n))
        self.assertEqual(hash(succ1n), hash(succ2n))
        self.assertFalse(succ1n == succ2n)
        self.assertTrue(succ1n.covers(succ2n) or succ2n.covers(succ1n))

        #pwlist = LatticeCoverPWList()
        #m = McReachability(succgen, goalchecker, pwlist)
        #init_state = succgen.get_initialstate()
        #pwlist.addState(init_state)
        #res = m.start()
        ##print pwlist.passed_waiting
        #self.assertEqual(5, len(pwlist.passed_waiting))

    def test_master_election_lattice3(self):
        file = open(os.path.join(os.path.dirname(__file__), 'master-election-simple3.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        s = GenerateSuccessorGenerator(nta)
        initial_state = s.get_initialstate()
        print initial_state

        succs = list(s.successors(initial_state))
        self.assertEqual(len(succs), 7)
        joined = copy.copy(succs[0])
        for succ in succs:
            print succ
            joined = joined.join(succ)
            print "joined: ", joined

        succs2 = list(s.successors(joined))
        self.assertEqual(len(succs2), 13)
        #for succ in succs2:
        #    print succ

        
    def test_lattice_cover_mc(self):
        file = open(os.path.join(os.path.dirname(__file__), 'master-election-simple3.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        succgen = GenerateSuccessorGenerator(nta)
        goalchecker = GoalChecker(succgen, 'E<> Node_0.master')

        #Node 0 can become master, cover pwlist
        pwlist = LatticeCoverPWList()
        m = McReachability(succgen, goalchecker, pwlist)
        init_state = succgen.get_initialstate()
        pwlist.addState(init_state)
        res = m.start()
        self.assertTrue(res)

    def test_lattice_join_mc(self):
        file = open(os.path.join(os.path.dirname(__file__), 'master-election-simple3.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        succgen = GenerateSuccessorGenerator(nta)
        goalchecker = GoalChecker(succgen, 'E<> Node_0.master')

        #Node 0 can become master, joining pwlist
        pwlist = LatticeJoinPWList()
        m = McReachability(succgen, goalchecker, pwlist)
        init_state = succgen.get_initialstate()
        pwlist.addState(init_state)
        res = m.start()
        self.assertTrue(res)

