#!/usr/bin/python
import sys
import os
import unittest
import logging
import subprocess

def hasWorkingLTSmin():
    try:
        subprocess.call(["opaal2lts-mc", '--version'], stdout=subprocess.PIPE)
        return True
    except OSError:
        pass
    return False

def hasApronLib():
    #XXX, other paths, other OS'?
    return (os.path.exists("/usr/lib/libapron.so") and
            os.path.exists("/usr/lib/liboctMPQ.so"))

@unittest.skipIf(not hasWorkingLTSmin(),
                     "need working LTSmin, skipping important tests!")
class TestLTSmin(unittest.TestCase):
    """Overall tests that invoke either LTSmin directly, or the opaal_ltsmin
    wrapper. These are high-level tests that test that "everything works".
    If errors occur frequently, consider adding more fine-grained tests.

    Requires a working LTSmin installation.
    """

    def setUp(self):
        self.opaal_ltsmin = os.path.join(os.path.dirname(__file__),"..", "bin", "opaal_ltsmin")

    def test_timed_ltl_viking1(self):
        """Tests DBM library, and invariant LTL query."""
        #get a .so file
        p = subprocess.Popen([self.opaal_ltsmin, "-v", 
            os.path.join(os.path.dirname(__file__), "vikingtests", "viking1.xml"),
            "--only-compile"],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        #work around s.t. nosetest can capture stdout
        (stdoutdata, stderrdata) = p.communicate()
        print stdoutdata
        self.assertEqual(p.returncode, 0)

        #invoke false LTL
        p = subprocess.Popen(["opaal2lts-mc",
                "--state=table",
                "-s 10",
                "--threads=1",
                "--strategy=cndfs",
                "--ltl=Viking1 == 1",
                "-u1",
                "viking1.so"],
                stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        #work around s.t. nosetest can capture stdout
        (stdoutdata, stderrdata) = p.communicate()
        print stdoutdata
        self.assertEqual(p.returncode, 0)

    @unittest.skipIf(not hasApronLib(),
                     "need Apron library (libapron-dev)")
    def test_octagon_fibcall(self):
        p = subprocess.Popen([self.opaal_ltsmin,
                "-v",
                os.path.join(os.path.dirname(__file__), "octagon_tests", "fibcall.xml")],
                stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        #work around s.t. nosetest can capture stdout
        (stdoutdata, stderrdata) = p.communicate()
        print stdoutdata
        self.assertEqual(p.returncode, 0)
