#!/usr/bin/python
import sys
import os
import unittest
import logging

import pyuppaal
from opaal.model_parsers.pyuppaal import SuccessorGenerator, GoalChecker
from opaal.passed_waiting_list import SimplePWList
from opaal.modelchecking_algorithms.reachability import McReachability

from opaal.lattices import DiscreteClock

class TestPYUPPAALTimeLattice(unittest.TestCase):

    def test_basic_discrete_clock_ops(self):
        c1 = DiscreteClock()
        c2 = DiscreteClock()

        self.assertEqual(0, int(c1))
        c1.delay()
        self.assertEqual(1, int(c1))

        self.assertTrue(c1 > c2)

        self.assertFalse(c1.covers(c2))
        self.assertFalse(c2.covers(c1))

        self.assertTrue(c2.delay())

        self.assertTrue(c1.covers(c2))
        self.assertTrue(c2.covers(c1))

        self.assertEqual(1, int(c2))
        c2.cmax = 2
        self.assertTrue(c2.delay())
        self.assertEqual(2, int(c2))
        self.assertTrue(c2.delay())
        self.assertEqual(3, int(c2))
        self.assertFalse(c2.delay())
        self.assertEqual(3, int(c2))
    
    @unittest.skip("Not working after refactoring - mchro will take a look at it later")
    def test_discrete_clock_simple1(self):
        file = open(os.path.join(os.path.dirname(__file__), 'discreteclock-simple1.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        succgen = SuccessorGenerator(nta)
        goalchecker = GoalChecker(succgen, 'E<> False')
        pwlist = SimplePWList()
        m = McReachability(succgen, goalchecker, pwlist)
        init_state = succgen.get_initialstate()
        pwlist.addState(init_state)
        res = m.start()

        self.assertEqual(pwlist.lenOfPWList(), 19)

        goalchecker = GoalChecker(succgen, 'E<> P1.goal and c >= 5')
        pwlist = SimplePWList()
        m = McReachability(succgen, goalchecker, pwlist)
        init_state = succgen.get_initialstate()
        pwlist.addState(init_state)
        res = m.start()
        self.assertTrue(res)

        goalchecker = GoalChecker(succgen, 'E<> P1.goal and c < 5')
        pwlist = SimplePWList()
        m = McReachability(succgen, goalchecker, pwlist)
        init_state = succgen.get_initialstate()
        pwlist.addState(init_state)
        res = m.start()
        self.assertFalse(res)

    def test_set_max_constant(self):
        c1 = DiscreteClock()
        c1.set_max_constant(2)
        c1.delay()
        c1.delay()
        c1.delay()
        self.assertEqual(int(c1), 3)

        c1.set_max_constant(1)
        self.assertEqual(int(c1), 2)
        c1.delay()
        self.assertEqual(int(c1), 2)

        c1.set_max_constant(0)
        self.assertEqual(int(c1), 1)
        c1.delay()
        self.assertEqual(int(c1), 1)

        #inactive
        c1.set_max_constant(-1)
        self.assertEqual(int(c1), 0)
        c1.delay()
        self.assertEqual(int(c1), 0)



if __name__ == '__main__':
    verbose_level = sys.argv.count('-v')
    logging.basicConfig( stream=sys.stderr, 
            level={0: logging.WARNING, 1: logging.INFO, 2: logging.DEBUG}[verbose_level] )

    unittest.main() 

# vim:ts=4:sw=4:expandtab

