#!/usr/bin/python
import sys
import os
import unittest
import logging
import tempfile
import pyuppaal
from ctypes import *
import ctypes
import copy
import pydbm
from opaal.model_parsers.pyuppaal import util
from opaal.model_parsers.pyuppaal.gensuccgen import GenerateSuccessorGenerator
from opaal.model_parsers.pyuppaal.simplify import SimplifyModel
from nose.tools import nottest


class TestPYUPPAALSimplify(unittest.TestCase):
    def test_simplify_template_param_var(self):
        file = open(os.path.join(os.path.dirname(__file__), 'simplify_template_param_var.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        s = GenerateSuccessorGenerator(nta)
        s1 = s.get_initialstate()

        # initial state
        self.assertEqual(s1.Process_x, 2)

        succs = list(s.successors(s1))

        self.assertEqual(len(succs), 1)
        s1 = succs[0]
        self.assertEqual(s1.Process_x, 3)

    def test_simplify_template_param_var_typedef(self):
        file = open(os.path.join(os.path.dirname(__file__), 'simplify_template_param_var_typedef.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        s = GenerateSuccessorGenerator(nta)
        s1 = s.get_initialstate()

        print s1

        # initial state
        self.assertEqual(s1.Template_0_x, 0)
        self.assertEqual(s1.Template_1_x, 1)

        succs = list(s.successors(s1))

        self.assertEqual(len(succs), 2)
        s1 = succs[0]
        self.assertEqual(s1.Template_0_x, 1)
        self.assertEqual(s1.Template_1_x, 1)
        
        s2 = succs[1]
        self.assertEqual(s2.Template_0_x, 0)
        self.assertEqual(s2.Template_1_x, 0)

    def test_simplify_template_param_var_typedef2(self):
        file = open(os.path.join(os.path.dirname(__file__), 'simplify_template_param_var_typedef2.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        s = GenerateSuccessorGenerator(nta)
        s1 = s.get_initialstate()

        print s1

        # initial state
        self.assertEqual(s1.P0_x, 0)
        self.assertEqual(s1.P1_x, 1)

        succs = list(s.successors(s1))

        self.assertEqual(len(succs), 2)
        s1 = succs[0]
        self.assertEqual(s1.P0_x, 1)
        self.assertEqual(s1.P1_x, 1)
        
        s2 = succs[1]
        self.assertEqual(s2.P0_x, 0)
        self.assertEqual(s2.P1_x, 0)

    def test_simplify_template_param_const(self):
        file = open(os.path.join(os.path.dirname(__file__), 'simplify_template_param_const.xml'))
        nta = pyuppaal.NTA.from_xml(file)
        
        s = GenerateSuccessorGenerator(nta)
        s1 = s.get_initialstate()
        succs = list(s.successors(s1))
        self.assertEqual(len(succs), 0)
    
if __name__ == '__main__':
    verbose_level = sys.argv.count('-v')
    logging.basicConfig( stream=sys.stderr, 
            level={0: logging.WARNING, 1: logging.INFO, 2: logging.DEBUG}[verbose_level] )

    unittest.main() 

#vim ts=4:sw=4:expandtab
